<?php

/**
 * This file contains \QUI\RabbitMQServer\Console
 */

namespace QUI\RabbitMQServer\Console;

use QUI;
use QUI\Exception;
use QUI\RabbitMQServer\HeartbeatService;
use QUI\RabbitMQServer\Interface\TaskSchedulerInterface;
use QUI\RabbitMQServer\Provider;
use QUI\System\License;

use function date;

/**
 * Different console options for RabbitMQ module
 *
 * @author Patrick Müller (www.pcsg.de)
 */
class ConnectionTest extends QUI\System\Console\Tool
{
    private HeartbeatService $heartbeatService;

    /**
     * @param TaskSchedulerInterface|null $taskScheduler
     *
     * @throws Exception
     */
    public function __construct()
    {
        $this->heartbeatService = new HeartbeatService();

        $this->setName('quiqqer:rabbitmqserver')
            ->setDescription('Operations for RabbitMQ QUIQQER module');

        $this->addArgument(
            'action',
            'Action to execute: connectiontest, tasktest'
        );

        $this->addArgument(
            'timeout',
            'Timeout in seconds for heartbeat test. (Default: 10 seconds)',
            false,
            true
        );
    }

    /**
     * (non-PHPdoc)
     *
     * @see \QUI\System\Console\Tool::execute()
     */
    public function execute(): void
    {
        switch ($this->getArgument('action')) {
            case 'connectiontest':
                $this->connectionTest();
                break;

            case 'tasktest':
                $this->taskTest();
                break;

            default:
                $this->writeLn('Unknown action. Please only use: connectiontest');
        }

        $this->writeLn("\nFinished.\n");
    }

    private function connectionTest(): void
    {
        $this->writeLn(
            "Testing connection to RabbitMQ server with current settings..."
        );

        try {
            $server = (new Provider())->getQueueServer();
            $server->getChannel();
        } catch (\Exception $Exception) {
            $this->write(" [error] :: " . $Exception->getMessage());
            return;
        }

        $this->write(" [ok]");
    }

    /**
     * Test task system.
     *
     * @return void
     */
    private function taskTest(): void
    {
        $this->writeLn(
            "Testing connection with test task to RabbitMQ task system with current settings..."
        );

        $testIdenfitier = QUI\Utils\Uuid::get();
        $timeoutSeconds = $this->getArgument('timeout') ? (int)$this->getArgument('timeout') : 10;

        $this->writeLn("Test task with heartbeat identifier {$testIdenfitier} successfully scheduled!");
        $this->writeLn("Checking heartbeat log for answer from worker...");

        $heartbeatFound = $this->heartbeatService->checkHeartbeat($testIdenfitier, $timeoutSeconds);

        if ($heartbeatFound) {
            $this->writeLn("Heartbeat found! Everything is OK!");
        } else {
            $this->writeLn("Heartbeat NOT found (timeout)!");
            $this->writeLn("Please check INFO log files in this QUIQQER system and in the task system.");
            $this->writeLn("\nThe logs should include the QUIQQER system ID: " . License::getSystemId());
            $this->write(
                "\nWhere to check for logs and possible errors:"
                . "\n - QUIQQER log files: rabbitmq_tasks.info-" . date('Y-m-d') . ".log"
                . "\n - Task system log files: Check the log dir configured in the config.json of all connected task systems"
                . " (default: var/log/)"
                . "\n - Syslog of the task system service (systemctl status [task-service-name].service)"
            );
        }
    }
}
